﻿using System;
using System.Linq;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Problems.DataAnalysis.Symbolic;
using HeuristicLab.Random;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace HeuristicLab.EvolutionTracking.Tests {
  [TestClass]
  public class QueryMatchPerformanceTest {
    private const int MaxDepth = 12;
    private const int MaxLength = 50;
    private const int N = 100000;


    [TestMethod]
    public void TestQueryMatchPerformance() {
      var grammar = new TypeCoherentExpressionGrammar();
      grammar.ConfigureAsDefaultRegressionGrammar();
      var creator = new ProbabilisticTreeCreator();
      var random = new MersenneTwister();

      var trees = Enumerable.Range(0, N).Select(x => creator.CreateTree(random, grammar, MaxLength, MaxDepth)).ToList();
      var comparer = new SymbolicExpressionTreeNodeEqualityComparer {
        MatchConstantValues = false,
        MatchVariableWeights = false,
        MatchVariableNames = true
      };
      var qm = new QueryMatch(comparer) { MatchParents = true };

      int count = trees.Skip(1).Count(x => qm.Match(x, trees[0]));

      Console.WriteLine("Count: " + count);
    }
  }
}
